#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

from plugins.generic.connector import Connector as GenericConnector


class Connector(GenericConnector):
    pass
