#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

from plugins.generic.filesystem import Filesystem as GenericFilesystem


class Filesystem(GenericFilesystem):
    pass
